package br.com.siq.gqfweb.anexo;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.AutoCreate;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.Transactional;
import org.richfaces.event.UploadEvent;

import br.com.siq.gqfweb.app.web.DownloadGateway;
import br.com.siq.gqfweb.app.web.GerenciadorExibicaoModalPanel;
import br.com.siq.gqfweb.app.web.HtmlDataGrid;
import br.com.siq.gqfweb.infraestrutura.PreviousUri;
import br.com.siq.gqfweb.infraestrutura.SiqCapability;
import br.com.siq.gqfweb.infraestrutura.annotation.ThrowsException;
import br.com.siq.gqfweb.sistema.ExceptionEntidadeNaoSalvaNoBanco;
import br.com.siq.gqfweb.sistema.ParametroRepositorio;
import br.com.siq.gqfweb.util.Mensagens;

@Name("gerenciadorAnexo")
@Scope(ScopeType.CONVERSATION)
@AutoCreate
@ThrowsException
public class GerenciadorAnexo {

	private List<Anexo> anexosNaoSalvos;
	
	private List<Anexo> anexosComProblemas; 
	
	private String anexosComProblemasConcatenados;
	
	private List<Anexo> listaAnexo;
	
	private HtmlDataGrid dataGrid;
	
	@In
	private PreviousUri previousUri;
	
	private String paginaRedirecionamento;
	
	@In
	private DownloadGateway download;
	
	private Anexo anexoSelecionado;
	
	private Anexo anexoSelecionadoRemocao;
	
	private List dadosSelecionadoRemocao;
	
	private int indiceAnexoSelecionadoRemocao;
	
	@In
	private ParametroRepositorio parametroRepositorio;
	
	@In
	private Mensagens mensagens;
	
	/*tamanho maximo permitido para todos os anexos*/
	private Integer tamanhoMaximoPermitidoAnexo;
	
	@In
	private Locale locale;
	
	
	/*Atributos Necessarios para o Anexo Generico*/
	private String grupoAnexoTipoGerenciado;
	
	private Object objetoComAnexo;
	
	private String direito;
	
	private boolean permitirAnexar;
	
	@In
	private GerenciadorExibicaoModalPanel gerenciadorExibicaoModalPanel;

	// private GrupoAnexoTipo grupoAnexoTipo;

	// private GrupoAnexo grupoAnexo;

	// private Collection<GrupoAnexo> grupoAnexos;

	@In
	private AnexoRepository anexoRepository;
	
	
	
	
	public boolean isPermitirAnexar() {
		return permitirAnexar;
	}

	public void setPermitirAnexar(boolean permitirAnexar) {
		this.permitirAnexar = permitirAnexar;
	}

	public String getDireito() {
		return direito;
	}

	public void setDireito(String direito) {
		this.direito = direito;
	}

	public String getGrupoAnexoTipoGerenciado() {
		return grupoAnexoTipoGerenciado;
	}

	public void setGrupoAnexoTipoGerenciado(String grupoAnexoTipoGerenciado) {
		this.grupoAnexoTipoGerenciado = grupoAnexoTipoGerenciado;
	}

	public Object getObjetoComAnexo() {
		return objetoComAnexo;
	}

	public void setObjetoComAnexo(Object objetoComAnexo) {
		this.objetoComAnexo = objetoComAnexo;
	}

	@In(required=false)
	private HtmlDataGrid htmlDataGrid;

	public List<Anexo> getAnexosNaoSalvos() {
		return anexosNaoSalvos;
	}

	public void setAnexosNaoSalvos(List<Anexo> anexosNaoSalvos) {
		this.anexosNaoSalvos = anexosNaoSalvos;
	}

	public String verificaExistemAnexosComProblemas(){
		
		if(getAnexosComProblemas().size()>0){
			
			anexosComProblemasConcatenados = "";
			
			for(int i=0; i<getAnexosComProblemas().size(); i++){
				
				if(anexosComProblemasConcatenados.equals("")){
					
					anexosComProblemasConcatenados += getAnexosComProblemas().get(i).getNome();
					
				}else{
					
					anexosComProblemasConcatenados += ", " + getAnexosComProblemas().get(i).getNome();
				}
			}
			
			this.setAnexosComProblemas(null);
			
			return "Richfaces.showModalPanel('panelErroAnexoGenerico')";
		}

		return null;
	}
	
	public void anexar(UploadEvent event) throws IOException {
		
		if(anexosNaoSalvos == null){
			anexosNaoSalvos = new ArrayList<Anexo>();
		}
		Anexo anexo = new Anexo();
		
		
		String arquivoNome = event.getUploadItem().getFileName();
		//se o nome contiver '/', ignorar e soh adotar o resto como nome
		if(arquivoNome.lastIndexOf("/")>0){
			anexo.setNome(arquivoNome.substring(arquivoNome.lastIndexOf("/")+1));
			
		}else if(arquivoNome.lastIndexOf("\\")>0){
			anexo.setNome(arquivoNome.substring(arquivoNome.lastIndexOf("\\")+1));
		}else{
			anexo.setNome(arquivoNome);
		}
		
		anexo.setDocumento(event.getUploadItem().getData());
		anexo.setTamanho(event.getUploadItem().getFileSize());
		
		if(anexo.getTamanho()!=null && getTamanhoMaximoPermitidoAnexo()!=null && (anexo.getTamanho()/1024)>getTamanhoMaximoPermitidoAnexo()){
				
			
			getAnexosComProblemas().add(anexo);

		}else{
			
			//grupoAnexoTipo = GrupoAnexoTipo.valueOf(nomeGrupoAnexoTipo);
			anexosNaoSalvos.add(anexo);
		}
	}
	
	
	public void anexar(Anexo _anexo) throws IOException {
		anexosNaoSalvos = new ArrayList<Anexo>();
		Anexo anexo = new Anexo();

		if(_anexo.getTamanho()!=null && getTamanhoMaximoPermitidoAnexo()!=null && (_anexo.getTamanho()/1024)>getTamanhoMaximoPermitidoAnexo()){
			getAnexosComProblemas().add(_anexo);
		}else{
			
			String arquivoNome = _anexo.getNome();
			//se o nome contiver '/', ignorar e soh adotar o resto como nome
			if(arquivoNome.lastIndexOf("/")>0){
				anexo.setNome(arquivoNome.substring(arquivoNome.lastIndexOf("/")+1));
				
			}else if(arquivoNome.lastIndexOf("\\")>0){
				anexo.setNome(arquivoNome.substring(arquivoNome.lastIndexOf("\\")+1));
			}else{
				anexo.setNome(arquivoNome);
			}
			
			anexo.setDocumento(_anexo.getDocumento());
			anexo.setTamanho(_anexo.getTamanho());
			//grupoAnexoTipo = GrupoAnexoTipo.valueOf(nomeGrupoAnexoTipo);
			anexosNaoSalvos.add(anexo);
		}
	}
	
	public List<Anexo> buscarGrupoAnexo(String nomeGrupoAnexoTipo,Object entidade){
		SiqCapability objetoComID = (SiqCapability)entidade;
		
		List<Anexo> anexosX = null;
		if(objetoComID!=null && objetoComID.getId()!= null ){		
			GrupoAnexoTipo grupoTipo = GrupoAnexoTipo.valueOf(nomeGrupoAnexoTipo);
			GrupoAnexo grupoAnexo = anexoRepository.obtemGrupoAnexo(grupoTipo, entidade);
			anexosX = grupoAnexo.getAnexos();
		}
		return anexosX;
	}	
	
	@Transactional
	public void salvar(String nomeGrupoAnexoTipo,Object entidade){
		for(Anexo anexoTemp:anexosNaoSalvos){
			anexoRepository.adiciona(anexoTemp, GrupoAnexoTipo.valueOf(nomeGrupoAnexoTipo), entidade);
		}
		anexosNaoSalvos = null;
	}
	

	@Transactional
	public void atualizar(Anexo anexo){
		anexoRepository.atualiza(anexo);
	}
	
	
	//DEPRECATED TOTAL (NAO REMOVE PARA MAIS DE 1 ANEXOS NUMA MESMA PAGINA)
	@Transactional
	public void remover(){
		List dados =(List) htmlDataGrid.getElement().getValue();
		anexoRepository.remove((Anexo)dados.get(htmlDataGrid.getElement().getRowIndex()));
		dados.remove(htmlDataGrid.getElement().getRowIndex());
	}
	
	
	//SOLUCAO REMOVER DINAMICAMENTE PELO RICH HTMLDATAGRID
	@Transactional
	public void remover(org.richfaces.component.html.HtmlDataGrid htmlDataGrid){
		List dados =(List) htmlDataGrid.getValue();
		anexoRepository.remove((Anexo)dados.get(htmlDataGrid.getRowIndex()));
		dados.remove(htmlDataGrid.getRowIndex());
	}
	
	
	
	@Transactional
	public void selecionarAnexoRemover(org.richfaces.component.html.HtmlDataGrid htmlDataGrid){
		dadosSelecionadoRemocao =(List) htmlDataGrid.getValue();
		anexoSelecionadoRemocao = ((Anexo)dadosSelecionadoRemocao.get(htmlDataGrid.getRowIndex()));
		indiceAnexoSelecionadoRemocao = htmlDataGrid.getRowIndex();
		
	}
	
	@Transactional
	public void selecionarAnexoRemoverForaDaPagina(Anexo anexo){
		anexoSelecionadoRemocao = anexo;
	}
	
	public void removerAnexoSelecionado(){
		
		anexoRepository.remove(anexoSelecionadoRemocao);
		dadosSelecionadoRemocao.remove(indiceAnexoSelecionadoRemocao);
		
		anexoSelecionadoRemocao = null;
		dadosSelecionadoRemocao = null;
		indiceAnexoSelecionadoRemocao = 0;
		
						
		
	}

	public void removerAnexoSelecionadoForaDaPagina(){
		
		anexoRepository.remove(anexoSelecionadoRemocao);
	}

	
	
	public void voltarPopupRemocao(){
		
		anexoSelecionadoRemocao = null;
		dadosSelecionadoRemocao = null;
		indiceAnexoSelecionadoRemocao = 0;
		
		
	}
	
	
	
	
	public void download(Anexo anexo){
		download.doDownload(anexo.getNome(),anexo.getDocumento());
	}
	

	public HtmlDataGrid getDataGrid() {
		return dataGrid;
	}

	public void setDataGrid(HtmlDataGrid dataGrid) {
		this.dataGrid = dataGrid;
	}	
	
	public Object exibirImagem(Anexo anexo) {

		Object resposta;
		
		
		if (anexo.isImage() != null) {
			
			resposta = anexo.getDocumento();
			
		}else{
			
			resposta = this.getClass().getClassLoader().getResourceAsStream("br/com/siq/gqfweb/resource/file.jpg");
			
		}		
		
		return resposta;
	}

	public String verificarTipoArquivoAnexoContentType(Anexo anexo) {

		String tipoArquivo;

		if (anexo.isImage() != null) {

			tipoArquivo = "image/" + anexo.isImage();
			
		} else {
			
			tipoArquivo = "image/jpg";
		}
		return tipoArquivo;
	}
	
	

	public String geraTelaAnexo(String idPanel, String identificador, String componente, String grupoAnexo){
		
		return "<rich:modalPanel id=\""+idPanel+"\"  width=\"700\" height=\"400\">"+
        	"<f:facet name=\"header\">" +
	            "<h:outputText value=\"#{messages.anexos_anexo}\"/>" +
    	    "</f:facet>	"	+
    		"<f:facet name=\"controls\">" +
    			"<h:outputText value=\"x\" style=\"cursor:pointer\" onclick=\"Richfaces.hideModalPanel('"+idPanel+"')\" />"+
    		"</f:facet> "+

	   "<a4j:form id=\"formUpload\"> " 	+	  	    
    	    "<h:panelGrid columns=\"1\" columnClasses=\"panelColunaTopo,panelColunaTopo\">"+
    	   
			"<rich:fileUpload id=\"uploadBox\"  addControlLabel=\"#{messages.adicionarArquivo_anexo}\" "+
       			 "stopEntryControlLabel=\"#{messages.cancelarUpload_anexo}\" uploadControlLabel=\"#{messages.uploadArquivo_anexo}\" "+
       			 "clearAllControlLabel=\"#{messages.limparTodos_anexo}\" clearControlLabel=\"#{messages.limpar_anexo}\" style=\"white-space:nowrap\""+
       			 "fileUploadListener=\"#{gerenciadorAnexo.anexar}\"  listHeight=\"125\" listWidth=\"650\""+
       			 "maxFilesQuantity=\"50\" immediateUpload=\"false\" >  "+
       			 
       			 "<a4j:support event=\"onuploadcomplete\" reRender=\"anexoInfo\""+
       			 	"action=\"#{gerenciadorAnexo.salvar('"+grupoAnexo+"', "+componente+")}\" />"+
       			 
        		 "<f:facet name=\"label\">"+
          	       "<h:outputText value=\"{_KB}KB from {MB}MB uploaded --- {mm}:{ss}\" />"+
        		 "</f:facet>"+
        		 
			"</rich:fileUpload>"+
			
	
		  "<h:panelGroup id=\"anexoInfo\">"+
			"<rich:panel bodyClass=\"anexoInfo\">"+
              "<f:facet name=\"header\">"+
                 "<h:outputText value=\"#{messages.arquivosAnexados_anexo}\"/>"+
              "</f:facet>"+
              
			"<rich:dataGrid columns=\"3\" width=\"600\" id=\"anexados\" binding=\"#{htmlDataGrid.element}\""+
				"value=\"#{gerenciadorAnexo.buscarGrupoAnexo('"+grupoAnexo+"', "+componente+")}\" "+
				"var=\"arquivo\" >"+
					"<rich:panel bodyClass=\"rich-laguna-panel-no-header\">		"	+	
                       
                                      	 "<h:outputText  value=\"#{arquivo.nome}\" />"+
		                               	 	" <h:panelGrid columns=\"2\" >"+
				                              
				                              "<h:outputText value=\"#{messages.descricao_anexo}\" />"+
				                              "<h:outputText   value=\"#{arquivo.descricao}\"  />"+
				                              "<h:outputText value=\"#{messages.tamanho_anexo}\" /> "  +                            	
				                             " <h:outputText value=\"#{arquivo.tamanho} bytes\" />"	+	
				                              
				                            "</h:panelGrid>	   " +                           	
	                               
	                               "#{gerenciadorAnexo.remover}\"  reRender=\"anexados\"  value=\"#{messages.apagar}\" />"+
                               	"<h:commandButton action=\"#{gerenciadorAnexo.download(arquivo)}\"    value=\"#{messages.download_anexo}\" />	"+
							                               	                              	
                          
                     "</rich:panel>"+
			"</rich:dataGrid> "   +
			          
			"</rich:panel>"+
		"</h:panelGroup>"	+
		"</h:panelGrid>"+
		"</a4j:form>"+
		"</rich:modalPanel>";
		
	}

	public Anexo getAnexoSelecionado() {
		return anexoSelecionado;
	}

	public void setAnexoSelecionado(Anexo anexoSelecionado) {
		this.anexoSelecionado = anexoSelecionado;
	}
	
	public void selecionaAnexo(Anexo anexoSelecionado) {
		this.anexoSelecionado = anexoSelecionado;
	}
	
	public Long buscaQuantidadeDeAnexos(Long id, String entidadeNome, String grupoAnexoTipo ){		
		
		Object entidade = anexoRepository.buscaEntidadeComAnexo(id, entidadeNome);	
		entidade = anexoRepository.refreshEntidade(entidade);
		GrupoAnexoTipo grupoTipo = GrupoAnexoTipo.valueOf(grupoAnexoTipo);		
		GrupoAnexo grupoAnexo = anexoRepository.obtemGrupoAnexo(grupoTipo, entidade);
		return grupoAnexo.quantidadeDeAnexos();
		
	}

	public Integer getTamanhoMaximoPermitidoAnexo() {
		
		if(this.tamanhoMaximoPermitidoAnexo==null){

			String tamanhoMaximoPermitidoAnexoString = parametroRepositorio.getParametroPelaChave("MAX_TAMANHO_ANEXO").getValor();
			
			if(tamanhoMaximoPermitidoAnexoString!=null && !tamanhoMaximoPermitidoAnexoString.equals("")){
				
				try{
					
					this.tamanhoMaximoPermitidoAnexo = Integer.valueOf(tamanhoMaximoPermitidoAnexoString);
					
				}catch(Exception e){
					
					e.printStackTrace();
				}
			}
		}
		
		return tamanhoMaximoPermitidoAnexo;
	}

	public void setTamanhoMaximoPermitidoAnexo(Integer tamanhoMaximoPermitidoAnexo) {
		this.tamanhoMaximoPermitidoAnexo = tamanhoMaximoPermitidoAnexo;
	}

	public List<Anexo> getAnexosComProblemas() {
		
		if(anexosComProblemas==null)
			anexosComProblemas = new ArrayList<Anexo>();
		
		return anexosComProblemas;
	}

	public void setAnexosComProblemas(List<Anexo> anexosComProblemas) {
		this.anexosComProblemas = anexosComProblemas;
	}

	
	
	public String getAnexosComProblemasConcatenados() {
		
		if(anexosComProblemasConcatenados==null)
			anexosComProblemasConcatenados="";
		
		return anexosComProblemasConcatenados;
	}

	public void setAnexosComProblemasConcatenados(
			String anexosComProblemasConcatenados) {
		this.anexosComProblemasConcatenados = anexosComProblemasConcatenados;
	}

	public String getMensagemFormatadaErro(){
		
		return mensagens.obterMensagemEmQualquerBundle("anexo_exception") + getAnexosComProblemasConcatenados(); 
		
	}
	/**
	 * 
	 * Mtodo necessrio para tornar o Anexo Genrico
	 * Parametros:
	 * 
	 * grupoAnexoTipo = Tipo do Grupo Anexo ex.: 'PAELABORACAO_ANEXO'
	 * objSelecionado = Objeto que ir conter os Anexos ex.: gerenciadorResultadoAvaliacao.resultadoAvaliacao
	 * direito = Direito especifico para anexar ex.: 'ANEXAR_RESULTADO_AVALICAO'
	 * permiteAnexar = Caso exista uma regra para exibir o anexo utilize este parametro ex.: gerenciadorPa.pa.isFinalizado
	 * 
	 * 
	 * @param grupoAnexoTipo
	 * @param objSelecionado
	 * @param direito
	 * @param permiteAnexar
	 */
	public String selecionarAnexos(String grupoAnexoTipo, Object objSelecionado, String direito, boolean permiteAnexar){	
		
		Method metodo = null;
		Object object = objSelecionado;
		Class classe =objSelecionado.getClass();
		
		try {

			metodo = classe.getMethod("getId");

			if(metodo.invoke(object,null) == null){
				throw new ExceptionEntidadeNaoSalvaNoBanco();
			}
		} catch (SecurityException e) {
			throw new RuntimeException(e);
		} catch (IllegalArgumentException e) {
			throw new RuntimeException(e);
		} catch (NoSuchMethodException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		} catch (InvocationTargetException e) {
			throw new RuntimeException(e);
		}
			

		
		this.grupoAnexoTipoGerenciado = grupoAnexoTipo;
		this.objetoComAnexo = objSelecionado;
		this.direito = direito;
		this.permitirAnexar = permiteAnexar;		
		gerenciadorExibicaoModalPanel.selecionaModalPanelAnexo();
		paginaRedirecionamento = previousUri.toString();
		
		return "/modulo/anexo/anexoPanel.xhtml";
		
	}

	public String voltar(){
		return paginaRedirecionamento;
	}
	
	public String getPaginaRedirecionamento() {
		return paginaRedirecionamento;
	}

	public void setPaginaRedirecionamento(String paginaRedirecionamento) {
		this.paginaRedirecionamento = paginaRedirecionamento;
	}
	
}
